﻿using UnityEngine;
using System.Collections;

public class Fading : MonoBehaviour
{

	public Texture2D fadeOutTexture_newLevel, fadeOutTexture_reload;    // the texture that will overlay the screen. This can be a black image or a loading graphic
	public float fadeSpeed_newLevel = 0.8f;      // the fading speed when loading a new level
	public float fadeSpeed_reload = 0.5f;      // the fading speed when reloading the same level

	private int drawDepth = -1000;      // the texture's order in the draw hierarchy: a low number means it renders on top
	private float alpha = 1.0f;         // the texture's alpha value between 0 and 1
	private int fadeDir = -1;           // the direction to fade: in = -1 or out = 1

	private bool loadingNewLevel = true;

	void OnGUI()
	{
		// fade out/in the alpha value using a direction, a speed and Time.deltaTime to convert the operation to seconds
		if (loadingNewLevel)
			alpha += fadeDir * fadeSpeed_newLevel * Time.deltaTime;
		else
			alpha += fadeDir * fadeSpeed_reload * Time.deltaTime;

		// force (clamp) the number to be between 0 and 1 because GUI.color uses Alpha values between 0 and 1
		alpha = Mathf.Clamp01(alpha);

		// set color of our GUI (in this case our texture). All color values remain the same & the Alpha is set to the alpha variable
		GUI.color = new Color(GUI.color.r, GUI.color.g, GUI.color.b, alpha);
		GUI.depth = drawDepth;                                                              // make the black texture render on top (drawn last)
		if (loadingNewLevel)
			GUI.DrawTexture(new Rect(0, 0, Screen.width, Screen.height), fadeOutTexture_newLevel);       // draw the texture to fit the entire screen area
		else
			GUI.DrawTexture(new Rect(0, 0, Screen.width, Screen.height), fadeOutTexture_reload);       // draw the texture to fit the entire screen area

	}

	// sets fadeDir to the direction parameter making the scene fade in if -1 and out if 1
	public float BeginFade(int direction)
	{
		fadeDir = direction;
		if (loadingNewLevel)
			return (fadeSpeed_newLevel);
		else
			return (fadeSpeed_reload);
	}

	// OnLevelWasLoaded is called when a level is loaded. It takes loaded level index (int) as a parameter so you can limit the fade in to certain scenes.
	void OnLevelWasLoaded()
	{
		alpha = 1;		// use this if the alpha is not set to 1 by default
		BeginFade(-1);      // call the fade in function
	}

	public void setLoadingNewLevel(bool value)
	{
		loadingNewLevel = value;
	}
}